# vim: noexpandtab:ts=4:sw=4
#
# This file is part of ReText
# Copyright: 2014 Maurice van der Pot
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest
from ReText import tablemode


class TestTableMode(unittest.TestCase):

	def performEdit(self, text, offset, editSize, paddingchar=None, fragment=None):
		if editSize < 0:
			text = text[:offset + editSize] + text[offset:]
		else:
			fragment = paddingchar * editSize if not fragment else fragment
			text = text[:offset] + fragment + text[offset:]
		return text

	def checkDetermineEditLists(self, paddingChars, before, edit, after, alignWithAnyEdge):
		class Row():
			def __init__(self, text, separatorLine, paddingChar):
				self.text = text
				self.separatorline = separatorLine
				self.paddingchar = paddingChar


		# Do some sanity checks on the test data to catch simple mistakes
		self.assertEqual(len(paddingChars), len(before),
		                 'The number of padding chars should be equal to the number of rows')
		self.assertEqual(len(before), len(after),
		                 'The number of rows before and after should be the same')
		# Apart from spacing edit only contains a's or d's
		self.assertTrue(edit[1].strip(' d') == '' or
		                edit[1].strip(' a') == '',
		                "An edit should be a sequence of a's or d's surrounded by spaces")

		rows = []
		for paddingChar, text in zip(paddingChars, before):
			rows.append(Row(text, (paddingChar != ' '), paddingChar))

		editedline = edit[0]
		editstripped = edit[1].strip()
		editsize = len(editstripped)

		# The offset passed to _determineEditLists is the one received from the
		# contentsChange signal and is always the start of the set of chars
		# that are added or removed.
		contentsChangeOffset = edit[1].index(editstripped[0])

		# However, the editoffset indicates the position before which chars
		# must be deleted or after which they must be added (just like the
		# offset used in the edits returned by _determineEditLists),
		# so for deletions we'll need to add the size of the edit to it
		if editstripped[0] == 'd':
			editsize = -editsize
			editoffset = contentsChangeOffset + len(editstripped)
		else:
			editoffset = contentsChangeOffset

		editLists = tablemode._determineEditLists(rows, edit[0], contentsChangeOffset, editsize, alignWithAnyEdge)


		editedRows = []

		self.assertEqual(len(editLists), len(rows))

		for i, (row, editList) in enumerate(zip(rows, editLists)):
			editedText = row.text

			for editEntry in editList:
				editedText = self.performEdit(editedText, editEntry[0], editEntry[1], paddingchar=row.paddingchar)

			editedRows.append(editedText)

		editedRows[editedline] = self.performEdit(editedRows[editedline], editoffset, editsize, fragment=editstripped)

		if editedRows != after:
			if alignWithAnyEdge:
				alignmentScenario = "when aligning any edge with another"
			else:
				alignmentScenario = "when only aligning edges of cells in the same column"

			assertMessage = ["Output differs %s." % alignmentScenario,
			                 "",
			                 "Input:"] + \
			                ["%3d '%s'" % (i, line) for i, line in enumerate(before)] + \
			                ["",
			                 "Edit:",
			                 "%3d '%s'" % edit,
			                 "",
			                 "Expected output:"] + \
			                ["%3d '%s'" % (i, line) for i, line in enumerate(after)] + \
			                ["",
			                 "Actual output:"] + \
			                ["%3d '%s'" % (i, line) for i, line in enumerate(editedRows)]

			self.fail('\n'.join(assertMessage))

	def test_simpleInsert(self):
		# Insert at the start of a cell so it doesn't need to grow
		separatorChars = '  '
		before  = ['|    |',
		           '|    |']

		edit = (0, ' a   ')

		after   = ['|a   |',
		           '|    |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert at the last position in a cell where it doesn't need to grow
		separatorChars = '  '
		before  = ['|    |',
		           '|    |']

		edit = (0, '    a')

		after   = ['|   a|',
		           '|    |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert at the end of a cell so it will have to grow
		separatorChars = '  '
		before  = ['|    |',
		           '|    |']

		edit = (0, '     a')

		after   = ['|    a|',
		           '|     |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

	def test_insertPushAhead(self):

		# Insert with enough room to push without growing the cell
		separatorChars = '  '
		before  = ['|  x |',
		           '|    |']

		edit = (0, ' a    ')

		after   = ['|a  x|',
		           '|    |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert without enough room to push, so the cell will have to grow
		separatorChars = '  '
		before  = ['|   x|',
		           '|    |']

		edit = (0, ' a    ')

		after   = ['|a   x|',
		           '|     |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)

		# Insert without enough room to push, so the cell will have to grow,
		# but the edge of the cell below it does not move with it because it is
		# of an earlier column
		separatorChars = '  '
		before  = ['| |   x|',
		           '  |    |']

		edit = (0, '   a    ')

		after   = ['| |a   x|',
		           '  |    |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert without enough room to push, so the cell will have to grow,
		# but the edge of the cell below it does not move with it because it is
		# of a later column
		separatorChars = '  '
		before  = ['  |   x|',
		           '| |    |']

		edit = (0, '   a    ')

		after   = ['  |a   x|',
		           '| |    |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert multiple characters forcing a partial grow
		separatorChars = '  '
		before  = ['|    |',
		           '|    |']

		edit = (0, '  aaaaaa')

		after   = ['| aaaaaa|',
		           '|       |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert multiple characters forcing a partial grow through pushing other chars ahead
		separatorChars = '  '
		before  = ['| bb   |',
		           '|      |']

		edit = (0, '  aaaaaaa')

		after   = ['| aaaaaaabb|',
		           '|          |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)


	def test_insertInSeparatorCell(self):

		# Insert in a cell on a separator line
		separatorChars = ' -'
		before  = ['|    |',
		           '|----|']

		edit = (1, '   a  ')

		after   = ['|    |',
		           '|--a-|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert in a cell on a separator line forcing it to grow
		separatorChars = ' -'
		before  = ['|    |',
		           '|----|']

		edit = (1, '    a ')

		after   = ['|     |',
		           '|---a-|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert in a cell on a separator line with an alignment marker
		separatorChars = ' -'
		before  = ['|    |',
		           '|---:|']

		edit = (1, '   a ')

		after   = ['|    |',
		           '|--a:|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert in a cell on a separator line with an alignment marker forcing it to grow
		separatorChars = ' -'
		before  = ['|    |',
		           '|---:|']

		edit = (1, '    a ')

		after   = ['|     |',
		           '|---a:|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert in a cell on a separator line after the alignment marker forcing it to grow
		separatorChars = ' -'
		before  = ['|    |',
		           '|---:|']

		edit = (1, '     a')

		after   = ['|     |',
		           '|---:a|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

	def test_insertAboveSeparatorLine(self):
		# Insert on another line, without growing the cell
		separatorChars = ' -'
		before  = ['|    |',
		           '|----|']

		edit = (0, '    a')

		after   = ['|   a|',
		           '|----|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert on another line, forcing the separator cell to grow
		separatorChars = ' -'
		before  = ['|    |',
		           '|----|']

		edit = (0, '     a')

		after   = ['|    a|',
		           '|-----|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert on another line, without growing the cell with alignment marker
		separatorChars = ' -'
		before  = ['|    |',
		           '|---:|']

		edit = (0, '    a')

		after   = ['|   a|',
		           '|---:|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert on another line, forcing the separator cell with alignment marker to grow
		separatorChars = ' -'
		before  = ['|    |',
		           '|---:|']

		edit = (0, '     a')

		after   = ['|    a|',
		           '|----:|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Insert on another line, forcing the separator cell that ends with a regular char to grow
		separatorChars = ' -'
		before  = ['|    |',
		           '|--- |']

		edit = (0, '     a')

		after   = ['|    a|',
		           '|---- |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

	def test_insertCascade(self):
		# Test if growing of cells cascades onto other lines through edges that are shifted
		separatorChars = '    '
		before  = ['|    |',
		           '     |    |',
		           '          |    |',
		           '     |']

		edit = (0, '     a')

		after   = ['|    a|',
		           '      |    |',
		           '           |    |',
		           '      |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)

		# Test if growing of cells cascades onto other lines but does not affect unconnected edges
		separatorChars = '   '
		before  = ['|    |',
		           '     |    |',
		           '       |  |    |']

		edit = (0, '     a')

		after   = ['|    a|',
		           '      |    |',
		           '       |   |    |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)

	def test_simpleDelete(self):
		# Delete at start of cell
		separatorChars = '  '
		before  = ['|abcd|',
		           '|    |']

		edit = (0, ' d   ')

		after   = ['|bcd|',
		           '|   |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Delete at end of cell
		separatorChars = '  '
		before  = ['|abcd|',
		           '|    |']

		edit = (0, '    d')

		after   = ['|abc|',
		           '|   |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

	def test_deleteShrinking(self):
		# Shrinking limited by cell on other row
		separatorChars = '  '
		before  = ['|abc |',
		           '|efgh|']

		edit = (0, ' d  ')

		after   = ['|bc  |',
		           '|efgh|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Shrinking limited by cell on other row (cont'd)
		separatorChars = '  '
		before  = ['|abcd|',
		           '|efgh|']

		edit = (0, '    d')

		after   = ['|abc |',
		           '|efgh|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Shrinking of next cell limited by cell on other row
		separatorChars = '  '
		before  = ['|abc |    |',
		           '|efghi|klm|']

		edit = (0, ' d  ')

		after   = ['|bc |     |',
		           '|efghi|klm|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Shrink current cell fully, grow next cell a partially
		separatorChars = '  '
		before  = ['| aabb|    |',
		           '|xxxxxx|x  |']

		edit = (0, '  dddd')

		after   = ['| |      |',
		           '|xxxxxx|x|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Shrink current cell fully, do not change next cell
		separatorChars = '  '
		before  = ['| aabb|    |',
		           '|xxxxxxxx  |']

		edit = (0, '  dddd')

		after   = ['| |    |',
		           '|xxxxxxxx  |']

		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

	def test_deleteShrinkingSeparatorRow(self):
		# Shrinking not limited by size of separator cell
		separatorChars = ' -'
		before  = ['|abcd|',
		           '|----|']

		edit = (0, '  d ')

		after   = ['|acd|',
		           '|---|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Shrinking limited by size of separator cell
		separatorChars = ' -'
		before  = ['|abc|',
		           '|---|']

		edit = (0, '  d  ')

		after   = ['|ac |',
		           '|---|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Shrinking not limited by size of separator cell with alignment markers
		separatorChars = ' -'
		before  = ['|abcd|',
		           '|:--:|']

		edit = (0, '  d ')

		after   = ['|acd|',
		           '|:-:|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Shrinking limited by size of separator cell with alignment markers
		separatorChars = ' -'
		before  = ['|abc|',
		           '|:-:|']

		edit = (0, '  d  ')

		after   = ['|ac |',
		           '|:-:|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

		# Shrinking partially limited by size of separator cell with alignment markers
		separatorChars = ' -'
		before  = ['|abcde|',
		           '|:---:|']

		edit = (0, '  dddd')

		after   = ['|a  |',
		           '|:-:|']

		self.checkDetermineEditLists(separatorChars, before, edit, after, True)
		self.checkDetermineEditLists(separatorChars, before, edit, after, False)

if __name__ == '__main__':
	unittest.main()



# vim: ts=8:sts=8:sw=8:noexpandtab
#
# This file is part of ReText
# Copyright: 2017-2021 Dmitry Shachnev
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from os.path import exists
import time
from PyQt5.QtCore import QDir, QUrl, Qt
from PyQt5.QtGui import QDesktopServices, QGuiApplication, QTextCursor, QTextDocument
from PyQt5.QtWidgets import QTextBrowser
from ReText import globalSettings

class ReTextPreview(QTextBrowser):

	def __init__(self, tab):
		QTextBrowser.__init__(self)
		self.tab = tab
		# if set to True, links to other files will unsuccessfully be opened as anchors
		self.setOpenLinks(False)
		self.anchorClicked.connect(self.openInternal)
		self.lastRenderTime = 0
		self.distToBottom = None
		self.verticalScrollBar().rangeChanged.connect(self.updateScrollPosition)

	def disconnectExternalSignals(self):
		pass

	def openInternal(self, link):
		url = link.url()
		if url.startswith('#'):
			self.scrollToAnchor(url[1:])
			return
		elif link.isRelative():
			fileToOpen = QDir.current().filePath(url)
		else:
			fileToOpen = link.toLocalFile() if link.isLocalFile() else None
		if fileToOpen is not None:
			if exists(fileToOpen):
				link = QUrl.fromLocalFile(fileToOpen)
				if globalSettings.handleWebLinks and fileToOpen.endswith('.html'):
					self.setSource(link)
					return
			# This is outside the "if exists" block because we can prompt for
			# creating the file
			if self.tab.openSourceFile(fileToOpen):
				return
		QDesktopServices.openUrl(link)

	def findText(self, text, flags, wrap=False):
		cursor = self.textCursor()
		if wrap and flags & QTextDocument.FindFlag.FindBackward:
			cursor.movePosition(QTextCursor.MoveOperation.End)
		elif wrap:
			cursor.movePosition(QTextCursor.MoveOperation.Start)
		newCursor = self.document().find(text, cursor, flags)
		if not newCursor.isNull():
			self.setTextCursor(newCursor)
			return True
		if not wrap:
			return self.findText(text, flags, wrap=True)
		return False

	def updateScrollPosition(self, minimum, maximum):
		"""Called when vertical scroll bar range changes.

		If this happened during preview rendering (less than 0.5s since it
		was started), set the position such that distance to bottom is the
		same as before refresh.
		"""
		timeSinceRender = time.time() - self.lastRenderTime
		if timeSinceRender < 0.5 and self.distToBottom is not None and maximum:
			newValue = maximum - self.distToBottom
			if newValue >= minimum:
				self.verticalScrollBar().setValue(newValue)


class ReTextWebPreview:
	"""This is a common class shared between WebKit and WebEngine
	based previews."""

	def __init__(self, editBox):
		self.editBox = editBox

		self.settings().setDefaultTextEncoding('utf-8')

		# Events relevant to sync scrolling
		self.editBox.cursorPositionChanged.connect(self._handleCursorPositionChanged)
		self.editBox.verticalScrollBar().valueChanged.connect(self.syncscroll.handleEditorScrolled)
		self.editBox.resized.connect(self._handleEditorResized)

		# Scroll the preview when the mouse wheel is used to scroll
		# beyond the beginning/end of the editor
		self.editBox.scrollLimitReached.connect(self._handleWheelEvent)

	def disconnectExternalSignals(self):
		self.editBox.cursorPositionChanged.disconnect(self._handleCursorPositionChanged)
		self.editBox.verticalScrollBar().valueChanged.disconnect(self.syncscroll.handleEditorScrolled)
		self.editBox.resized.disconnect(self._handleEditorResized)

		self.editBox.scrollLimitReached.disconnect(self._handleWheelEvent)

	def _handleCursorPositionChanged(self):
		editorCursorPosition = self.editBox.verticalScrollBar().value() + \
				       self.editBox.cursorRect().top()
		self.syncscroll.handleCursorPositionChanged(editorCursorPosition)

	def _handleEditorResized(self, rect):
		self.syncscroll.handleEditorResized(rect.height())

	def wheelEvent(self, event):
		if QGuiApplication.keyboardModifiers() == Qt.KeyboardModifier.ControlModifier:
			zoomFactor = self.zoomFactor()
			zoomFactor *= 1.001 ** event.angleDelta().y()
			self.setZoomFactor(zoomFactor)
		return super().wheelEvent(event)



#!/usr/bin/env python3

VERSION = '7.2.3'

long_description = '''\
ReText is simple text editor that supports Markdown and reStructuredText
markup languages. It is written in Python using PyQt libraries.

It supports live preview, tabs, math formulas, export to various formats
including PDF and HTML.

For more details, please go to the `home page`_ or to the `wiki`_.

.. _`home page`: https://github.com/retext-project/retext
.. _`wiki`: https://github.com/retext-project/retext/wiki'''

import os
import sys
from os.path import join, basename
from setuptools import setup, Command
from setuptools.command.sdist import sdist
from setuptools.command.install import install
from distutils import log
from distutils.command.build import build
from subprocess import check_call
from glob import glob, iglob

if sys.version_info[0] < 3:
	sys.exit('Error: Python 3.x is required.')

if len(sys.argv) > 1 and sys.argv[1] == 'bdist_wheel':
	sys.exit('Building wheels is disabled, because it breaks .desktop files. '
	         'See issues #452 and #497 for details.\n'
	         'If you are using pip, please ignore this error, installation '
	         'should still succeed.')

def bundle_icons():
	import urllib.request
	import tarfile
	from io import BytesIO
	icons_tgz = 'https://github.com/retext-project/retext/archive/icons.tar.gz'
	response = urllib.request.urlopen(icons_tgz)
	tario = BytesIO(response.read())
	tar = tarfile.open(fileobj=tario, mode='r')
	for member in tar:
		if member.isfile():
			member.path = basename(member.path)
			log.info('bundling icons/%s', member.path)
			tar.extract(member, 'icons')
	tar.close()


class retext_build_translations(Command):
	description = 'Build .qm files from .ts files using lrelease'
	user_options = []

	def initialize_options(self):
		pass

	def finalize_options(self):
		pass

	def run(self):
		environment = dict(os.environ, QT_SELECT='5')
		for ts_file in glob(join('locale', '*.ts')):
			try:
				check_call(('lrelease', ts_file), env=environment)
			except Exception as e:
				log.warn('Failed to build translations: %s', e)
				break


class retext_build(build):
	sub_commands = build.sub_commands + [('build_translations', None)]


class retext_sdist(sdist):
	def run(self):
		self.run_command('build_translations')
		bundle_icons()
		sdist.run(self)

class retext_install(install):
	def change_roots(self, *names):
		self.orig_install_scripts = self.install_scripts
		self.orig_install_data = self.install_data
		install.change_roots(self, *names)

	def run(self):
		install.run(self)

		if self.root is None:
			self.orig_install_scripts = self.install_scripts
			self.orig_install_data = self.install_data
		retext = join(self.orig_install_scripts, 'retext')

		# Fix Exec and Icon fields in the desktop file
		desktop_file_path = join(self.install_data, 'share', 'applications',
		                         'me.mitya57.ReText.desktop')
		icon_path = join(self.orig_install_data, 'share', 'retext', 'icons', 'retext.svg')
		with open(desktop_file_path, encoding="utf-8") as desktop_file:
			desktop_contents = desktop_file.read()
		log.info('fixing Exec line in %s', desktop_file_path)
		desktop_contents = desktop_contents.replace('Exec=retext', 'Exec=%s' % retext)
		if self.orig_install_data != '/usr':
			log.info('fixing Icon line in %s', desktop_file_path)
			desktop_contents = desktop_contents.replace('Icon=retext', 'Icon=%s' % icon_path)
		with open(desktop_file_path, 'w', encoding="utf-8") as desktop_file:
			desktop_file.write(desktop_contents)


classifiers = [
	'Development Status :: 5 - Production/Stable',
	'Environment :: X11 Applications :: Qt',
	'License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)',
	'Programming Language :: Python :: 3 :: Only',
	'Topic :: Text Editors',
	'Topic :: Text Processing :: Markup'
]

setup(name='ReText',
      version=VERSION,
      description='Simple editor for Markdown and reStructuredText',
      long_description=long_description,
      author='Dmitry Shachnev',
      author_email='mitya57@gmail.com',
      url='https://github.com/retext-project/retext',
      packages=['ReText'],
      entry_points={
        'gui_scripts': ['retext = ReText.__main__:main'],
      },
      data_files=[
        ('share/applications', ['data/me.mitya57.ReText.desktop']),
        ('share/icons/hicolor/scalable/apps', ['icons/retext.svg']),
        ('share/metainfo', ['data/me.mitya57.ReText.appdata.xml']),
        ('share/retext/icons', iglob('icons/*')),
        ('share/retext/locale', iglob('locale/*.qm'))
      ],
      python_requires='>=3.6',
      requires=['docutils', 'Markdown', 'Markups(>=2.0)', 'pyenchant', 'Pygments', 'PyQt5(>=5.11)'],
      install_requires=[
        'docutils',
        'Markdown>=3.0',
        'Markups>=2.0',
        'Pygments',
        'chardet>=2.3',
        'PyQt5>=5.11',
      ],
      extras_require={
        'spellcheck': ['pyenchant'],
      },
      cmdclass={
        'build_translations': retext_build_translations,
        'build': retext_build,
        'sdist': retext_sdist,
        'install': retext_install,
      },
      test_suite='tests',
      classifiers=classifiers,
      license='GPL 2+'
)



# vim: ts=8:sts=8:sw=8:noexpandtab

# This file is part of ReText
# Copyright: 2012-2021 Dmitry Shachnev
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import markups
import markups.common
from os.path import dirname, exists, join, expanduser

from PyQt5.QtCore import QByteArray, QLocale, QSettings, QStandardPaths
from PyQt5.QtGui import QFont, QFontDatabase

app_version = "7.2.3"

settings = QSettings('ReText project', 'ReText')

if not str(settings.fileName()).endswith('.conf'):
	# We are on Windows probably
	settings = QSettings(QSettings.Format.IniFormat, QSettings.Scope.UserScope,
		'ReText project', 'ReText')

datadirs = []

def initializeDataDirs():
	assert not datadirs

	try:
		datadirs.append(dirname(dirname(__file__)))
	except NameError:
		pass

	dataLocations = QStandardPaths.standardLocations(QStandardPaths.StandardLocation.GenericDataLocation)
	datadirs.extend(join(d, 'retext') for d in dataLocations)

	if sys.platform == "win32":
		# Windows compatibility: Add "PythonXXX\share\" path
		datadirs.append(join(dirname(sys.executable), 'share', 'retext'))

	# For virtualenvs
	datadirs.append(join(dirname(dirname(sys.executable)), 'share', 'retext'))

_iconPath = None

def getBundledIcon(iconName):
	global _iconPath
	if _iconPath is None:
		for dir in ['icons'] + datadirs:
			_iconPath = join(dir, 'icons')
			if exists(_iconPath):
				break
	return join(_iconPath, iconName + '.png')

configOptions = {
	'appStyleSheet': '',
	'autoSave': False,
	'defaultCodec': '',
	'defaultMarkup': markups.MarkdownMarkup.name,
	'defaultPreviewState': 'editor',
	'detectEncoding': True,
	'directoryPath': expanduser("~"),
	'documentStatsEnabled': False,
	'editorFont': QFont(),
	'font': QFont(),
	'handleWebLinks': False,
	'hideToolBar': False,
	'highlightCurrentLine': 'disabled',
	'iconTheme': '',
	'lastTabIndex': 0,
	'lineNumbersEnabled': False,
	'markdownDefaultFileExtension': '.mkd',
	'openFilesInExistingWindow': True,
	'openLastFilesOnStartup': False,
	'orderedListMode': 'increment',
	'paperSize': '',
	'pygmentsStyle': 'default',
	'recentDocumentsCount': 10,
	'relativeLineNumbers': False,
	'restDefaultFileExtension': '.rst',
	'rightMargin': 0,
	'rightMarginWrap': False,
	'saveWindowGeometry': False,
	'showDirectoryTree': False,
	'spellCheck': False,
	'spellCheckLocale': '',
	'styleSheet': '',
	'syncScroll': True,
	'tabBarAutoHide': False,
	'tabInsertsSpaces': True,
	'tabWidth': 4,
	'uiLanguage': QLocale.system().name(),
	'useFakeVim': False,
	'useWebEngine': False,
	'useWebKit': False,
	'wideCursor': False,
	'windowGeometry': QByteArray(),
	'windowTitleFullPath': False,
}

def readFromSettings(key, keytype, settings=settings, default=None):
	if isinstance(default, QFont):
		family = readFromSettings(key, str, settings, default.family())
		size = readFromSettings(key + 'Size', int, settings, 0)
		return QFont(family, size)
	if not settings.contains(key):
		return default
	try:
		value = settings.value(key, type=keytype)
		if isinstance(value, keytype):
			return value
		return keytype(value)
	except TypeError as error:
		# Type mismatch
		print('Warning: '+str(error))
		# Return an instance of keytype
		return default if (default is not None) else keytype()

def readListFromSettings(key, settings=settings):
	if not settings.contains(key):
		return []
	value = settings.value(key)
	if isinstance(value, str):
		return [value]
	else:
		return value

def writeToSettings(key, value, default, settings=settings):
	if isinstance(value, QFont):
		writeToSettings(key, value.family(), '', settings)
		writeToSettings(key + 'Size', max(value.pointSize(), 0), 0, settings)
	elif value == default:
		settings.remove(key)
	else:
		settings.setValue(key, value)

def writeListToSettings(key, value, settings=settings):
	if len(value) > 1:
		settings.setValue(key, value)
	elif len(value) == 1:
		settings.setValue(key, value[0])
	else:
		settings.remove(key)

def getSettingsFilePath(settings=settings):
	return settings.fileName()


class ReTextSettings(object):
	def __init__(self):
		for option in configOptions:
			value = configOptions[option]
			object.__setattr__(self, option, readFromSettings(
				option, type(value), default=value))

	def __setattr__(self, option, value):
		if not option in configOptions:
			raise AttributeError('Unknown attribute')
		object.__setattr__(self, option, value)
		writeToSettings(option, value, configOptions[option])

	def __getattribute__(self, option):
		value = object.__getattribute__(self, option)
		# Choose a font just-in-time, because when the settings are
		# loaded it is too early to work.
		if option == 'font' and not value.family():
			value = QFont()
		if option == 'editorFont' and not value.family():
			value = QFontDatabase.systemFont(QFontDatabase.SystemFont.FixedFont)
		return value

globalSettings = ReTextSettings()

markups.common.PYGMENTS_STYLE = globalSettings.pygmentsStyle
